@@ -20,10 +20,11 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <stdint.h>
 #include <string.h>
-#include <sys/utsname.h>
+#include <stdint.h>
 #include <sqlite3.h>
+#include <inttypes.h>
+#include <sys/utsname.h>
 #include "slog_internal.h"
 
 /*
@@ -43,7 +44,7 @@
 
 /**
  * insert_addl_data_os
- * @brief Helper fuction for inserting into the os table
+ * @brief Helper function for inserting into the os table
  *
  * @param[in] slog the servicelog to receive the event
  * @param[in] event the event to be inserted
@@ -52,9 +53,11 @@ int
 insert_addl_data_os(servicelog *slog, struct sl_event *event)
 {
 	int rc;
-	char buf[1024], *version;
+	char *version;
+	const char *out;
 	struct sl_data_os *os;
 	struct utsname uname_buf;
+	sqlite3_stmt *pstmt = NULL;
 
 	os = (struct sl_data_os *)event->addl_data;
 
@@ -65,19 +68,42 @@ insert_addl_data_os(servicelog *slog, struct sl_event *event)
 	else
 		version = os->version;
 
-	snprintf(buf, 1024, "INSERT OR REPLACE INTO os (event_id, version, subsystem, "
-		 "driver, device) VALUES (%llu, '%s', '%s', '%s', '%s');",
-		 event->id, version, os->subsystem, os->driver, os->device);
-	rc = sqlite3_exec(slog->db, buf, NULL, NULL, NULL);
+	rc = sqlite3_prepare(slog->db, "INSERT OR REPLACE INTO os (event_id,"
+		" version, subsystem, driver, device) VALUES (?, ?, ?, ?, ?);",
+		 -1, &pstmt, &out);
 	if (rc != SQLITE_OK)
 		return rc;
 
-	return SQLITE_OK;
+	rc = sqlite3_bind_int64(pstmt, 1, event->id);
+	rc = rc ? rc : sqlite3_bind_text(pstmt, 2, version,
+					 strlen(version), SQLITE_STATIC);
+	rc = rc ? rc : sqlite3_bind_text(pstmt, 3, os->subsystem,
+					 strlen(os->subsystem), SQLITE_STATIC);
+	rc = rc ? rc : sqlite3_bind_text(pstmt, 4, os->driver,
+					 strlen(os->driver), SQLITE_STATIC);
+	rc = rc ? rc : sqlite3_bind_text(pstmt, 5, os->device,
+					 strlen(os->device), SQLITE_STATIC);
+
+	if (rc != SQLITE_OK)
+		goto sqlt_fail;
+
+	rc = sqlite3_step(pstmt);
+	if (rc != SQLITE_ROW && rc != SQLITE_DONE)
+		goto sqlt_fail;
+
+	rc = sqlite3_finalize(pstmt);
+	return rc;
+
+sqlt_fail:
+	snprintf(slog->error, SL_MAX_ERR, "%s", sqlite3_errmsg(slog->db));
+	rc = sqlite3_finalize(pstmt);
+
+	return rc;
 }
 
 /**
  * insert_addl_data_rtas
- * @brief Helper fuction for inserting into the rtas table
+ * @brief Helper function for inserting into the rtas table
  *
  * @param[in] slog the servicelog to receive the event
  * @param[in] event the event to be inserted
@@ -85,34 +111,62 @@ insert_addl_data_os(servicelog *slog, struct sl_event *event)
 int
 insert_addl_data_rtas(servicelog *slog, struct sl_event *event)
 {
+	const char *out;
 	int rc;
-	char buf[1024];
 	struct sl_data_rtas *rtas;
+	sqlite3_stmt *pstmt = NULL;
 
 	rtas = (struct sl_data_rtas *)event->addl_data;
 
-	snprintf(buf, 1024, "INSERT OR REPLACE INTO rtas (event_id, action_flags, "
-		 "platform_id, creator_id, subsystem_id, pel_severity, "
-		 "event_type, event_subtype, kernel_id, addl_word1, "
-		 "addl_word2, addl_word3, addl_word4, addl_word5, addl_word6, "
-		 "addl_word7, addl_word8) VALUES (%llu, %u, %u, '%c', %u, %u, "
-		 "%u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u);", event->id,
-		 rtas->action_flags, rtas->platform_id, rtas->creator_id,
-		 rtas->subsystem_id, rtas->pel_severity, rtas->event_type,
-		 rtas->event_subtype, rtas->kernel_id, rtas->addl_words[0],
-		 rtas->addl_words[1], rtas->addl_words[2], rtas->addl_words[3],
-		 rtas->addl_words[4], rtas->addl_words[5], rtas->addl_words[6],
-		 rtas->addl_words[7]);
-	rc = sqlite3_exec(slog->db, buf, NULL, NULL, NULL);
+	rc = sqlite3_prepare(slog->db, "INSERT OR REPLACE INTO rtas ("
+		"event_id, action_flags, platform_id, creator_id,"
+		" subsystem_id, pel_severity, event_type, event_subtype,"
+		" kernel_id, addl_word1, addl_word2, addl_word3, addl_word4,"
+		" addl_word5, addl_word6, addl_word7, addl_word8) VALUES (?,"
+		" ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);",
+		-1, &pstmt, &out);
 	if (rc != SQLITE_OK)
 		return rc;
 
-	return SQLITE_OK;
+	rc = sqlite3_bind_int64(pstmt, 1, event->id);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 2, rtas->action_flags);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 3, rtas->platform_id);
+	rc = rc ? rc : sqlite3_bind_text(pstmt, 4, &(rtas->creator_id),
+					 1, SQLITE_STATIC);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 5, rtas->subsystem_id);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 6, rtas->pel_severity);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 7, rtas->event_type);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 8, rtas->event_subtype);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 9, rtas->kernel_id);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 10, rtas->addl_words[0]);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 11, rtas->addl_words[1]);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 12, rtas->addl_words[2]);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 13, rtas->addl_words[3]);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 14, rtas->addl_words[4]);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 15, rtas->addl_words[5]);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 16, rtas->addl_words[6]);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 17, rtas->addl_words[7]);
+
+	if (rc != SQLITE_OK)
+		goto sqlt_fail;
+
+	rc = sqlite3_step(pstmt);
+	if (rc != SQLITE_ROW && rc != SQLITE_DONE)
+		goto sqlt_fail;
+
+	rc = sqlite3_finalize(pstmt);
+	return rc;
+
+sqlt_fail:
+	snprintf(slog->error, SL_MAX_ERR, "%s", sqlite3_errmsg(slog->db));
+	rc = sqlite3_finalize(pstmt);
+
+	return rc;
 }
 
 /**
  * insert_addl_data_enclosure
- * @brief Helper fuction for inserting into the enclosure table
+ * @brief Helper function for inserting into the enclosure table
  *
  * @param[in] slog the servicelog to receive the event
  * @param[in] event the event to be inserted
@@ -120,25 +174,48 @@ insert_addl_data_rtas(servicelog *slog, struct sl_event *event)
 int
 insert_addl_data_enclosure(servicelog *slog, struct sl_event *event)
 {
+	const char *out;
 	int rc;
-	char buf[1024];
+	sqlite3_stmt *pstmt = NULL;
 	struct sl_data_enclosure *encl;
 
 	encl = (struct sl_data_enclosure *)event->addl_data;
 
-	snprintf(buf, 1024, "INSERT OR REPLACE INTO enclosure (event_id, enclosure_model, "
-		 "enclosure_serial) VALUES (%llu, '%s', '%s');", event->id,
-		 encl->enclosure_model, encl->enclosure_serial);
-	rc = sqlite3_exec(slog->db, buf, NULL, NULL, NULL);
-	if (rc != SQLITE_OK)
+	rc = sqlite3_prepare(slog->db, "INSERT OR REPLACE INTO enclosure"
+		" (event_id, enclosure_model, enclosure_serial) VALUES (?,"
+		" ?, ?);", -1, &pstmt, &out);
+	if (rc != SQLITE_OK) {
+		snprintf(slog->error, SL_MAX_ERR, "%s",
+				sqlite3_errmsg(slog->db));
 		return rc;
+	}
 
-	return SQLITE_OK;
+	rc = sqlite3_bind_int64(pstmt, 1, event->id);
+	rc = rc ? rc : sqlite3_bind_text(pstmt, 2, encl->enclosure_model,
+					 strlen(encl->enclosure_model), SQLITE_STATIC);
+	rc = rc ? rc : sqlite3_bind_text(pstmt, 3, encl->enclosure_serial,
+					 strlen(encl->enclosure_serial), SQLITE_STATIC);
+
+	if (rc != SQLITE_OK)
+		goto sqlt_fail;
+
+	rc = sqlite3_step(pstmt);
+	if (rc != SQLITE_ROW && rc != SQLITE_DONE)
+		goto sqlt_fail;
+
+	rc = sqlite3_finalize(pstmt);
+	return rc;
+
+sqlt_fail:
+	snprintf(slog->error, SL_MAX_ERR, "%s", sqlite3_errmsg(slog->db));
+	rc = sqlite3_finalize(pstmt);
+
+	return rc;
 }
 
 /**
  * insert_addl_data_bmc
- * @brief Helper fuction for inserting into the bmc table
+ * @brief Helper function for inserting into the bmc table
  *
  * @param[in] slog the servicelog to receive the event
  * @param[in] event the event to be inserted
@@ -146,29 +223,54 @@ insert_addl_data_enclosure(servicelog *slog, struct sl_event *event)
 int
 insert_addl_data_bmc(servicelog *slog, struct sl_event *event)
 {
+	const char *out;
 	int rc;
-	char buf[1024];
 	struct sl_data_bmc *bmc;
+	sqlite3_stmt *pstmt = NULL;
 
 	bmc = (struct sl_data_bmc *)event->addl_data;
 
-	snprintf(buf, 1024, "INSERT OR REPLACE INTO bmc (event_id, sel_id, sel_type, "
-		 "generator, version, sensor_type, sensor_number, event_class, "
-		 "event_type, direction) VALUES (%llu, %u, %u, %u, %u, %u, %u, "
-		 "%u, %u, %d);", event->id, bmc->sel_id, bmc->sel_type,
-		 bmc->generator, bmc->version, bmc->sensor_type,
-		 bmc->sensor_number, bmc->event_class, bmc->event_type,
-		 bmc->direction);
-	rc = sqlite3_exec(slog->db, buf, NULL, NULL, NULL);
-	if (rc != SQLITE_OK)
+	rc = sqlite3_prepare(slog->db, "INSERT OR REPLACE INTO bmc (event_id,"
+		" sel_id, sel_type, generator, version, sensor_type,"
+		" sensor_number, event_class, event_type, direction) VALUES"
+		" (?, ?, ?, ?, ?, ?, ?, ?, ?, ?);", -1, &pstmt, &out);
+	if (rc != SQLITE_OK) {
+		snprintf(slog->error, SL_MAX_ERR,
+			 "%s", sqlite3_errmsg(slog->db));
 		return rc;
+	}
 
-	return SQLITE_OK;
+	rc = sqlite3_bind_int64(pstmt, 1, event->id);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 2, bmc->sel_id);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 3, bmc->sel_type);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 4, bmc->generator);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 5, bmc->version);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 6, bmc->sensor_type);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 7, bmc->sensor_number);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 8, bmc->event_class);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 9, bmc->event_type);
+	rc = rc ? rc : sqlite3_bind_int(pstmt, 10, bmc->direction);
+
+	if (rc != SQLITE_OK)
+		goto sqlt_fail;
+
+	rc = sqlite3_step(pstmt);
+	if (rc != SQLITE_ROW && rc != SQLITE_DONE)
+		goto sqlt_fail;
+
+	rc = sqlite3_finalize(pstmt);
+	return rc;
+
+sqlt_fail:
+	snprintf(slog->error, SL_MAX_ERR, "%s", sqlite3_errmsg(slog->db));
+	rc = sqlite3_finalize(pstmt);
+
+	return SQLITE_INTERNAL;
 }
 
 /**
  * retrieve_addl_data_os
- * @brief Helper fuction for retrieving from the os table
+ * @brief Helper function for retrieving from the os table
  *
  * @param[in] e the event to be retrieved
  * @param[in] argc the number of columns in the table
@@ -182,36 +284,59 @@ retrieve_addl_data_os(void *e, int argc, char **argv, char **column)
 	struct sl_event *event = (struct sl_event *)e;
 	struct sl_data_os *os;
 
-	event->addl_data = malloc(sizeof(struct sl_data_os));
-	os = (struct sl_data_os *)event->addl_data;
+	event->addl_data = calloc(1, sizeof(struct sl_data_os));
+	if (!event->addl_data)
+		return 1;
 
-	memset(os, 0, sizeof(struct sl_data_os));
+	os = (struct sl_data_os *)event->addl_data;
 
 	for (i=0; i<argc; i++) {
 		if (!strcmp(column[i], "version")) {
-			os->version = malloc(strlen(argv[i]) + 1);
-			strcpy(os->version, argv[i]);
+			os->version = strdup(argv[i]);
+			if (!os->version)
+				goto free_mem;
 		}
 		else if (!strcmp(column[i], "subsystem")) {
-			os->subsystem = malloc(strlen(argv[i]) + 1);
-			strcpy(os->subsystem, argv[i]);
+			os->subsystem = strdup(argv[i]);;
+			if (!os->subsystem)
+				goto free_mem;
 		}
 		else if (!strcmp(column[i], "driver")) {
-			os->driver = malloc(strlen(argv[i]) + 1);
-			strcpy(os->driver, argv[i]);
+			os->driver = strdup(argv[i]);
+			if (!os->driver)
+				goto free_mem;
 		}
 		else if (!strcmp(column[i], "device")) {
-			os->device = malloc(strlen(argv[i]) + 1);
-			strcpy(os->device, argv[i]);
+			os->device = strdup(argv[i]);
+			if (!os->device)
+				goto free_mem;
 		}
 	}
 
 	return 0;
+
+free_mem:
+	if (os->version)
+		free(os->version);
+
+	if (os->subsystem)
+		free(os->subsystem);
+
+	if (os->driver)
+		free(os->driver);
+
+	if (os->device)
+		free(os->device);
+
+	if (event->addl_data)
+		free(event->addl_data);
+
+	return 1;
 }
 
 /**
  * retrieve_addl_data_rtas
- * @brief Helper fuction for retrieving from the rtas table
+ * @brief Helper function for retrieving from the rtas table
  *
  * @param[in] e the event to be retrieved
  * @param[in] argc the number of columns in the table
@@ -225,10 +350,11 @@ retrieve_addl_data_rtas(void *e, int argc, char **argv, char **column)
 	struct sl_event *event = (struct sl_event *)e;
 	struct sl_data_rtas *rtas;
 
-	event->addl_data = malloc(sizeof(struct sl_data_rtas));
-	rtas = (struct sl_data_rtas *)event->addl_data;
+	event->addl_data = calloc(1, sizeof(struct sl_data_rtas));
+	if (!event->addl_data)
+		return 1;
 
-	memset(rtas, 0, sizeof(struct sl_data_rtas));
+	rtas = (struct sl_data_rtas *)event->addl_data;
 
 	for (i=0; i<argc; i++) {
 		if (!strcmp(column[i], "action_flags")) {
@@ -286,7 +412,7 @@ retrieve_addl_data_rtas(void *e, int argc, char **argv, char **column)
 
 /**
  * retrieve_addl_data_enclosure
- * @brief Helper fuction for retrieving from the enclosure table
+ * @brief Helper function for retrieving from the enclosure table
  *
  * @param[in] e the event to be retrieved
  * @param[in] argc the number of columns in the table
@@ -300,28 +426,43 @@ retrieve_addl_data_enclosure(void *e, int argc, char **argv, char **column)
 	struct sl_event *event = (struct sl_event *)e;
 	struct sl_data_enclosure *encl;
 
-	event->addl_data = malloc(sizeof(struct sl_data_enclosure));
-	encl = (struct sl_data_enclosure *)event->addl_data;
+	event->addl_data = calloc(1, sizeof(struct sl_data_enclosure));
+	if (!event->addl_data)
+		return 1;
 
-	memset(encl, 0, sizeof(struct sl_data_enclosure));
+	encl = (struct sl_data_enclosure *)event->addl_data;
 
 	for (i=0; i<argc; i++) {
 		if (!strcmp(column[i], "enclosure_serial")) {
-			encl->enclosure_serial = malloc(strlen(argv[i]) + 1);
-			strcpy(encl->enclosure_serial, argv[i]);
+			encl->enclosure_serial = strdup(argv[i]);
+			if (!encl->enclosure_serial)
+				goto free_mem;
 		}
 		else if (!strcmp(column[i], "enclosure_model")) {
-			encl->enclosure_model = malloc(strlen(argv[i]) + 1);
-			strcpy(encl->enclosure_model, argv[i]);
+			encl->enclosure_model = strdup(argv[i]);
+			if (!encl->enclosure_model)
+				goto free_mem;
 		}
-	}
+	} /* for */
 
 	return 0;
+
+free_mem:
+	if (encl->enclosure_serial)
+		free(encl->enclosure_serial);
+
+	if (encl->enclosure_model)
+		free(encl->enclosure_model);
+
+	if (event->addl_data)
+		free(event->addl_data);
+
+	return 1;
 }
 
 /**
  * retrieve_addl_data_bmc
- * @brief Helper fuction for retrieving from the bmc table
+ * @brief Helper function for retrieving from the bmc table
  *
  * @param[in] e the event to be retrieved
  * @param[in] argc the number of columns in the table
@@ -335,10 +476,11 @@ retrieve_addl_data_bmc(void *e, int argc, char **argv, char **column)
 	struct sl_event *event = (struct sl_event *)e;
 	struct sl_data_bmc *bmc;
 
-	event->addl_data = malloc(sizeof(struct sl_data_bmc));
-	bmc = (struct sl_data_bmc *)event->addl_data;
+	event->addl_data = calloc(1, sizeof(struct sl_data_bmc));
+	if (!event->addl_data)
+		return 1;
 
-	memset(bmc, 0, sizeof(struct sl_data_bmc));
+	bmc = (struct sl_data_bmc *)event->addl_data;
 
 	for (i=0; i<argc; i++) {
 		if (!strcmp(column[i], "sel_id")) {
@@ -375,7 +517,7 @@ retrieve_addl_data_bmc(void *e, int argc, char **argv, char **column)
 
 /**
  * free_addl_data_os
- * @brief Helper fuction for freeing an event that was retrieved
+ * @brief Helper function for freeing an event that was retrieved
  *
  * @param[in] event the event containing an additional data section to be freed
  */
@@ -396,7 +538,7 @@ void free_addl_data_os(struct sl_event *event)
 
 /**
  * free_addl_data_rtas
- * @brief Helper fuction for freeing an event that was retrieved
+ * @brief Helper function for freeing an event that was retrieved
  *
  * @param[in] event the event containing an additional data section to be freed
  */
@@ -412,7 +554,7 @@ void free_addl_data_rtas(struct sl_event *event)
 
 /**
  * free_addl_data_enclosure
- * @brief Helper fuction for freeing an event that was retrieved
+ * @brief Helper function for freeing an event that was retrieved
  *
  * @param[in] event the event containing an additional data section to be freed
  */
@@ -432,7 +574,7 @@ void free_addl_data_enclosure(struct sl_event *event)
 
 /**
  * free_addl_data_bmc
- * @brief Helper fuction for freeing an event that was retrieved
+ * @brief Helper function for freeing an event that was retrieved
  *
  * @param[in] event the event containing an additional data section to be freed
  */
@@ -645,7 +787,7 @@ print_addl_data_rtas(FILE *str, struct sl_event *event, int verbosity)
 			break;
 		case 0x23:
 			detail = " - Predictive Error, fault may be corrected "
-				"after boot, degraded performace";
+				"after boot, degraded performance";
 			break;
 		case 0x24:
 			detail = " - Predictive Error, loss of redundancy";
@@ -731,6 +873,7 @@ print_addl_data_rtas(FILE *str, struct sl_event *event, int verbosity)
 		case 0xE0:
 			detail = " - Platform powered off by user without "
 				"normal shutdown (abnormal power-off)";
+			break;
 		default:
 			detail = "";
 	}
