@@ -19,6 +19,7 @@
 #include <malloc.h>
 #include <errno.h>
 #include <time.h>
+#include <inttypes.h>
 #include "slog_internal.h"
 #define EXCLUDE_SERVICELOG_COMPAT_DECLS
 #include "../servicelog-1/libservicelog.h"
@@ -63,7 +64,7 @@ convert_key_to_v29(servicelog *log, uint64_t key64, uint32_t *key32,
 	*key32 = (uint32_t) key64;
 	if (*key32 != key64) {
 		snprintf(log->error, SL_MAX_ERR,
-			"v1+ %s ID %lld truncated to 32 bits", key_type, key64);
+			"v1+ %s ID ""%"PRIu64 "truncated to 32 bits", key_type, key64);
 		return EDOM;
 	}
 	return 0;
@@ -310,7 +311,7 @@ convert_rtas_to_v29(servicelog *log, struct sl_event *ev, void **data29,
 	ev29_sz = sizeof(rtas29) + description_sz +
 		nr_callouts*sizeof(struct sl_ppc64_callout) +
 		ev->raw_data_len;
-	ev29 = (struct sl_ppc64_rtas*) malloc(ev29_sz);
+	ev29 = malloc(ev29_sz);
 	if (!ev29) {
 		snprintf(log->error, SL_MAX_ERR, "out of memory in %s",
 							__FUNCTION__);
@@ -393,7 +394,7 @@ convert_encl_to_v29(servicelog *log, struct sl_event *ev, void **data29,
 	ev29_sz = sizeof(encl29) + description_sz +
 		nr_callouts*sizeof(struct sl_ppc64_callout) +
 		ev->raw_data_len;
-	ev29 = (struct sl_ppc64_encl *) malloc(ev29_sz);
+	ev29 = malloc(ev29_sz);
 	if (!ev29) {
 		snprintf(log->error, SL_MAX_ERR, "out of memory in %s",
 							__FUNCTION__);
@@ -406,7 +407,7 @@ convert_encl_to_v29(servicelog *log, struct sl_event *ev, void **data29,
 	next += sizeof(encl29);
 
 	if (description_sz) {
-		strcpy(next, ev->description);
+		strncpy(next, ev->description, (ev29_sz - sizeof(encl29) - 1));
 		ev29->description = next;
 		next += description_sz;
 	}
@@ -486,9 +487,9 @@ static int
 find_repaired_events(servicelog *log, uint64_t repair_id,
 						struct sl_event **events)
 {
-	char query[40];
+	char query[40] = {0,};
 
-	sprintf(query, "repair=%lld", repair_id);
+	snprintf(query, 39, "repair=""%" PRIu64, repair_id);
 	return servicelog_event_query(log, query, events);
 }
 
@@ -512,6 +513,7 @@ convert_v1_repair_to_v29(servicelog *log, struct sl_repair_action *rpr1,
 	char *next;
 	size_t ev29_sz;
 	struct sl_event *re, *repaired_events = NULL;
+	int str_size;
 
 	if (find_repaired_events(log, rpr1->id, &repaired_events) != 0)
 		repaired_events = NULL;
@@ -546,7 +548,7 @@ convert_v1_repair_to_v29(servicelog *log, struct sl_repair_action *rpr1,
 
 	ev29_sz = sizeof(rpr29) + location_sz + procedure_sz +
 		rpr29.num_repairs*sizeof(uint32_t);
-	ev29 = (struct sl_repair *) malloc(ev29_sz);
+	ev29 = malloc(ev29_sz);
 	if (!ev29) {
 		snprintf(log->error, SL_MAX_ERR, "out of memory in %s",
 							__FUNCTION__);
@@ -559,13 +561,15 @@ convert_v1_repair_to_v29(servicelog *log, struct sl_repair_action *rpr1,
 	next = (char*) ev29;
 	next += sizeof(rpr29);
 
+	str_size = (ev29_sz - sizeof(rpr29) - 1);
 	if (location_sz) {
-		strcpy(next, rpr1->location);
+		strncpy(next, rpr1->location, str_size);
 		ev29->location = next;
 		next += location_sz;
+		str_size -= (location_sz + 1);
 	}
 	if (procedure_sz) {
-		strcpy(next, rpr1->procedure);
+		strncpy(next, rpr1->procedure, str_size);
 		ev29->procedure = next;
 		next += procedure_sz;
 	}
@@ -697,8 +701,8 @@ v29_types_to_v1_match(char *next, uint64_t bitmap)
 	for (t = SL_TYPE_OS; t <= SL_TYPE_PPC64_ENCL; t++) {
 		if (bitmap & (1 << t)) {
 			if (++ntypes > 1)
-				next += sprintf(next, " OR ");
-			next += sprintf(next, "type == %d",
+				next += snprintf(next, 5, " OR ");
+			next += snprintf(next, 10, "type == %d",
 						convert_type_to_v1(t));
 		}
 	}
@@ -723,7 +727,7 @@ int
 convert_v29_query_to_v1(struct sl_query *v29_query, char **v1_where)
 {
 	char tmp[1024];
-	char *next = tmp;
+	char *next = tmp, *end = (tmp + sizeof(tmp) - 1);
 	char *and_connector = " AND ";
 	char *connector = "";
 
@@ -742,38 +746,41 @@ convert_v29_query_to_v1(struct sl_query *v29_query, char **v1_where)
 	// Create time boundaries
 	if (v29_query->start_time) {
 		char d[32];
-		next += sprintf(next, "%stime_event >= '%s'", connector,
-			encode_db_date(d, 32, v29_query->start_time));
+		next += snprintf(next, (end - next), "%stime_event >= '%s'",
+				connector,
+				encode_db_date(d, 32, v29_query->start_time));
 		connector = and_connector;
 	}
 
 	if (v29_query->end_time) {
 		char d[32];
-		next += sprintf(next, "%stime_event <= '%s'", connector,
+		next += snprintf(next, (end - next), "%stime_event <= '%s'",
+			connector,
 			encode_db_date(d, 32, v29_query->end_time));
 		connector = and_connector;
 	}
 
 	if (v29_query->is_serviceable == SL_QUERY_YES) {
-		next += sprintf(next, "%sserviceable = 1", connector);
+		next += snprintf(next, (end - next), "%sserviceable = 1",
+			connector);
 		connector = and_connector;
 	} else if (v29_query->is_serviceable == SL_QUERY_NO) {
-		next += sprintf(next, "%sserviceable = 0", connector);
+		next += snprintf(next, (end - next), "%sserviceable = 0",
+			connector);
 		connector = and_connector;
 	}
 
 	if (v29_query->is_repaired == SL_QUERY_YES) {
-		next += sprintf(next, "%srepair != 0", connector);
+		next += snprintf(next, (end - next), "%srepair != 0", connector);
 		connector = and_connector;
 	} else if (v29_query->is_repaired == SL_QUERY_NO) {
-		next += sprintf(next, "%srepair = 0", connector);
+		next += snprintf(next, (end - next), "%srepair = 0", connector);
 		connector = and_connector;
 	}
 
 	if (v29_query->severity > 1) {
-		next += sprintf(next, "%sseverity >= %d", connector,
-						v29_query->severity);
-		connector = and_connector;
+		next += snprintf(next, (end - next), "%sseverity >= %d",
+			connector, v29_query->severity);
 	}
 
 	*v1_where = strdup(tmp);
@@ -834,7 +841,7 @@ query_repair_events(servicelog *log, struct sl_query *v29_query,
 {
 	int result;
 	char v1_match[1024];
-	char *next = v1_match;
+	char *next = v1_match, *end = (next + sizeof(v1_match) - 1);
 	char *connector = "";
 	struct sl_repair_action *v1, *v1_repairs;
 	struct sl_header *v29, *v29_repairs, **v29_next;
@@ -849,24 +856,27 @@ query_repair_events(servicelog *log, struct sl_query *v29_query,
 				|| v29_query->is_repaired == SL_QUERY_YES
 				|| v29_query->severity > SL_SEV_INFO)
 		return 0;
-	
+
 	/* Find all the repair_actions in the specified time range. */
 	*next = '\0';
 	if (v29_query->start_time) {
 		char d[32];
-		next += sprintf(next, "%stime_repair >= '%s'", connector,
+		next += snprintf(next, (end - next), "%stime_repair >= '%s'",
+			connector,
 			encode_db_date(d, 32, v29_query->start_time));
 		connector = " AND ";
 	}
 	if (v29_query->end_time) {
 		char d[32];
-		next += sprintf(next, "%stime_repair <= '%s'", connector,
+		next += snprintf(next, (end - next), "%stime_repair <= '%s'",
+			connector,
 			encode_db_date(d, 32, v29_query->end_time));
 	}
 
 	v1_repairs = NULL;
 	result = servicelog_repair_query(log, v1_match, &v1_repairs);
 	if (result != 0) {
+		servicelog_repair_free(v1_repairs);
 		/* servicelog_repair_query() set log->error. */
 		return result;
 	}
@@ -1057,8 +1067,8 @@ _convert_v1_sl_notify_to_v29(struct v29_servicelog *slog, struct v29_sl_notify *
 		v29nfy_gram_fini();
 		if (parse_result != 0 || semantic_errors != 0) {
 			snprintf(log->error, SL_MAX_ERR, "can't translate "
-				"match string '%s' for v1+ sl_notify %llu "
-				"to v0.2.9 sl_notify", v1->match, v1->id);
+				"match string '%s' for v1+ sl_notify ""%" PRIu64 "to "
+				"v0.2.9 sl_notify", v1->match, v1->id);
 			return ENOTSUP;
 		}
 	 }
@@ -1077,7 +1087,7 @@ _convert_v1_sl_notify_to_v29(struct v29_servicelog *slog, struct v29_sl_notify *
 	v29->method = v1->method;	// These happen to match.
 	v29->command_length = size - sizeof(struct v29_sl_notify);
 	cmd = ((char *) v29) + sizeof(struct v29_sl_notify);
-	strcpy(cmd, v1->command);
+	strncpy(cmd, v1->command, v29->command_length);
 
 	return 0;
 }
@@ -1125,7 +1135,7 @@ int v29_servicelog_notify_query(struct v29_servicelog *slog, char *command,
 			// Calc total mem needed to store v29_sl_notify struct + data on end (command and such)
 			size = sizeof(struct v29_sl_notify)
 						+ strlen(v1->command) + 1;
-			v29 = (struct v29_sl_notify*) malloc(size);
+			v29 = malloc(size);
 			if (!v29) {
 				snprintf(log->error, SL_MAX_ERR,
 					"out of memory in %s", __FUNCTION__);
@@ -1178,7 +1188,7 @@ int v29_servicelog_notify_get(struct v29_servicelog *slog, uint32_t id,
 		return rc;
 
 	size = sizeof(struct v29_sl_notify) + strlen(v1->command) + 1;
-	v29 = (struct v29_sl_notify*) malloc(size);
+	v29 = malloc(size);
 	if (!v29) {
 		snprintf(log->error, SL_MAX_ERR, "out of memory in %s",
 							__FUNCTION__);
@@ -1211,7 +1221,7 @@ _convert_v29_sl_notify_to_v1(servicelog *log, struct v29_sl_notify *v29,
 						struct sl_notify *v1)
 {
 	char v1_match[1024];
-	char *next;
+	char *next = v1_match, *end  = (next + sizeof(v1_match) - 1);
 	char *connector = "";
 	char *and_connector = " AND ";
 
@@ -1237,16 +1247,16 @@ _convert_v29_sl_notify_to_v1(servicelog *log, struct v29_sl_notify *v29,
 	if (next > v1_match)
 		connector = and_connector;
 	if (v29->severity > 1) {
-		next += sprintf(next, "%sseverity >= %d", connector,
-							v29->severity);
+		next += snprintf(next, (end - next), "%sseverity >= %d",
+			connector, v29->severity);
 		connector = and_connector;
 	}
 	switch (v29->serviceable_event) {
 	case SL_QUERY_YES:
-		sprintf(next, "%sserviceable == 1", connector);
+		snprintf(next, (end - next), "%sserviceable == 1", connector);
 		break;
 	case SL_QUERY_NO:
-		sprintf(next, "%sserviceable == 0", connector);
+		snprintf(next, (end - next), "%sserviceable == 0", connector);
 		break;
 	}
 	v1->match = strdup(v1_match);
